#include <errno.h>			/* Needed for errno */

#ifdef	NO_ERRNO
extern	int	errno;
#endif

#include <stdio.h>

#ifndef	NO_STDLIB
#include <stdlib.h>
#endif

#ifndef	NO_STRING_H
#include <string.h>
#endif

#include <time.h>			/* strftime */

#ifndef	NO_UNISTD_H
#include <unistd.h>			/* Flock flags */
#endif

#include <ctype.h>			/* isspace */

#ifndef	NO_MALLOC_H
#include <malloc.h>			/* malloc */
#endif

#include <sys/file.h>
#include <sys/stat.h>			/* umask */
